#! /usr/bin/env python3 -B

from testlib import *
import json
import program01

#####################################################################################

def test_compress_image(name, expected):
    "\nSi comprime l'immagine name.png con la funzione compress_image\n"
    image        = name + '.png'
    compressed   = 'test-' + name + '-compressed.png'
    explanation1 = "Il numero di coppie inserite nella immagine compressa non e' quello atteso"
    file_expected= 'ris-' + name + '-compressed.png'
    ris = program01.compress_image(image, compressed)
    check(ris, expected, None, explanation1)
    check_img_file(compressed, file_expected)
    return 2

def test_compress_image_1():
    "\nSi comprime l'immagine 3cime.png con la funzione compress_image\n"
    name         = '3cime'
    expected     = 41992
    return test_compress_image(name, expected)

def test_compress_image_2():
    "\nSi comprime l'immagine scacchiera.png con la funzione compress_image\n"
    name         = 'scacchiera'
    expected     = 8600
    return test_compress_image(name, expected)

def test_compress_image_3():
    "\nSi comprime l'immagine blue-specks.png con la funzione compress_image\n"
    name         = 'blue-specks'
    expected     = 8423 # 8467
    return test_compress_image(name, expected)

#####################################################################################

def test_decompress_image(name, expected):
    "\nSi decomprime l'immagine name-compressed.png con la funzione decompress_image\n"
    image        = name + '-compressed.png'
    decompressed = 'test-' + name + '-decompressed.png'
    explanation1 = "La larghezza della immagine decompressa non e' quella attesa"
    file_expected= 'ris-' + name + '-decompressed.png'
    ris = program01.decompress_image(image, decompressed)
    check(ris, expected, None, explanation1)
    check_img_file(decompressed, file_expected)
    return 2

def test_decompress_image_1():
    "\nSi decomprime l'immagine 3cime-compressed.png con la funzione decompress_image\n"
    name         = '3cime'
    expected     = 275
    return test_decompress_image(name,expected)

def test_decompress_image_2():
    "\nSi decomprime l'immagine scacchiera-compressed.png con la funzione decompress_image\n"
    name         = 'scacchiera'
    expected     = 300
    return test_decompress_image(name,expected)

def test_decompress_image_3():
    "\nSi decomprime l'immagine blue-specks-compressed.png con la funzione decompress_image\n"
    name         = 'blue-specks'
    expected     = 1001
    return test_decompress_image(name,expected)

#####################################################################################

def test_histogramJson(name, expected):
    "\nSi crea l'istogramma come JSON name-histogram.json con la funzione color_histogram\n"
    image        = name + '.png'
    histojson    = 'test-' + name + '-histogram.json'
    file_expected= 'ris-'  + name + '-histogram.json'
    explanation1 = "Il numero di colori individuato non e' quello atteso"
    explanation2 = "L'istogramma calcolato e' diverso da quello atteso"
    ris = program01.color_histogram(image, histojson)
    check(ris, expected, None, explanation1)
    with open(file_expected) as exf: dict_expected = json.load(exf)
    with open(histojson) as hf:      dict_ris = json.load(hf)
    check(dict_ris, dict_expected, None, explanation2)
    return 1

def test_histogramJson_1():
    "\nSi crea l'istogramma come JSON 3cime-histogram.json con la funzione color_histogram\n"
    name         = '3cime'
    expected     = 26983
    return test_histogramJson(name, expected)

def test_histogramJson_2():
    "\nSi crea l'istogramma come JSON scacchiera-histogram.json con la funzione color_histogram\n"
    name         = 'scacchiera'
    expected     = 4
    return test_histogramJson(name, expected)

def test_histogramJson_3():
    "\nSi crea l'istogramma come JSON blue-specks-histogram.json con la funzione color_histogram\n"
    name         = 'blue-specks'
    expected     = 5
    return test_histogramJson(name, expected)

#####################################################################################

def test_histogramPNG(name, expected):
    "\nSi crea l'istogramma come immagine name-histogram.png con la funzione paint_histogram\n"
    histojson    = name + '-histogram.json'
    image        = 'test-' + name + '-histogram.png'
    explanation1 = "La larghezza dell'istogramma PNG non e' quella attesa"
    file_expected= 'ris-' + name + '-histogram.png'
    ris = program01.paint_histogram(histojson,image)
    check(ris, expected, None, explanation1)
    check_img_file(image, file_expected)
    return 2

def test_histogramPNG_1():
    "\nSi crea l'istogramma come immagine 3cime-histogram.png con la funzione paint_histogram\n"
    name         = '3cime'
    expected     = 113
    return test_histogramPNG(name, expected)

def test_histogramPNG_2():
    "\nSi crea l'istogramma come immagine scacchiera-histogram.png con la funzione paint_histogram\n"
    name = 'scacchiera'
    expected     = 39200
    return test_histogramPNG(name, expected)

def test_histogramPNG_3():
    "\nSi crea l'istogramma come immagine blue-specks-histogram.png con la funzione paint_histogram\n"
    name         = 'blue-specks'
    expected     = 920062
    return test_histogramPNG(name, expected)

#####################################################################################

tests = [
    test_compress_image_1,      test_compress_image_2,      test_compress_image_3,
    test_decompress_image_1,                                test_decompress_image_3,
    test_histogramJson_1,       test_histogramJson_2,       test_histogramJson_3,
    test_histogramPNG_1,        test_histogramPNG_2,
]

if __name__ == '__main__':
    runtests(tests,logfile='grade01.csv')
