'''
Si devono manipolare delle immagini in formato PNG disponibili su file.

Potete usare le librerie immagini e json per caricare e salvare file PNG e JSON.

Esercizio 1:
Si progetti la funzione compress_image(fileImageIn, fileCompressed) che legge una immagine PNG e la trasforma in formato compresso usando il seguente schema:
Per ogni linea dell'immagine ogni gruppo di N<256 pixel consecutivi dello stesso colore (R,G,B) viene rappresentato/sostituito dalla coppia di pixel (R, G, B), (0, 0, N).
Esempio: la riga [ (255,0,0), (255, 0, 0), (255, 0, 0), (0, 0, 255) ] viene rappresentata dalla riga [ (255,0,0),(0,0,3), (0,0,255),(0,0,1) ].
Il valore N non puo' quindi essere maggiore di 255, e se una sequenza di colori uguali consecutivi e' piu' lunga di 255, 
la si deve spezzare in tanti gruppi da 255 seguiti da un gruppo contenente il resto dei pixel.
Per fare in modo che tutte le linee della immagine compressa abbiano la stessa lunghezza, a ciascuna linea viene aggiunto un 'padding' (riempimento) 
formato da tanti pixel (255, 255, 255) quanti bastano a rendere le linee uguali.
La funzione, una volta letto e decodificato il file PNG fileImageIn, deve salvare nel file 'fileCompressed' l'immagine che si ottiene usando lo schema precedente.
La funzione inoltre ritorna il numero totale di gruppi (coppie) individuato.

Esercizio 2:
si progetti la funzione decompress_image(fileCompressed, fileImageOut) che:
- legge da fileCompressed una immagine PNG costruita secondo lo schema precedente e che ricostruisce l'immagine originale
    - eliminando da ciascuna linea il padding formato dai pixel (255, 255, 255) in fondo alle linee se presenti
    - espandendo le coppie (R, G, B), (0, 0, N) nei gruppi originali di N pixel (R,G,B) consecutivi uguali
- salva l'immagine nel fileImageOut
La funzione ritorna la larghezza dell'immagine ricreata.

Esercizio 3:
si progetti la funzione color_frequence(fileImageIn, fileJsonOut) che calcola la frequenza dei colori presenti nella immagine fileImageIn e la salva in formato JSON.
Le frequenze risultanti da salvare in formato JSON sono rappresentate sotto forma di un dizionario in cui:
- le chiavi sono stringhe nel formato "R;G;B" dei soli colori presenti nella immagine (esempio: il colore rosso (255,0,0) -> "255;0;0")
- i valori sono il numero di pixel con quel colore, presenti nella immagine
La funzione inoltre ritorna il numero di colori diversi individuati.

Esercizio 4:
si progetti la funzione paint_histogram(fileJsonIn, fileImageOut) che 
crea e salva in formato PNG l'istogramma delle frequenze dei colori letto dal file JSON fileJsonIn (nello stesso formato dell'esercizio 3).
NOTA: ciascuna tupla (R,G,B) di interi che definisce un colore deve essere riottenuta dalle chiavi del dizionario letto dal file.
   Esempio:    "100;200;10" -> (100, 200, 10).
L'immagine ha un numero di linee uguali al numero di colori presenti nel dizionario
Le linee devono essere ordinate in ordine crescente di colore (in ordine di canale, cioe' prima in ordine di R, poi di G poi di B).
Ciascuna linea e' formata da un numero di pixel dello stesso colore, pari al valore presente nel dizionario.
Per rendere l'immagine rettangolare, a ciascuna linea se necessario devono essere aggiunti pixel neri (0, 0, 0)
in modo tale che tutte le linee abbiano la stessa (minima) lunghezza.
L'immagine risultante va salvata in formato PNG nel file fileImageOut.
La funzione inoltre ritorna la larghezza dell'immagine creata.

NOTA: non usate caratteri accentati (neanche nei commenti)
'''

import immagini
import json

########################################################################################

def compress_image(fileImageIn, fileCompressed):
    '''Mettete il vostro codice qui'''
    img = immagini.load(fileImageIn)
    w = len(img[0])
    h = len(img)
    compressed = []
    maxlen = 0
    numpairs = 0
    for line in img:
        cline = []
        last = line[0]
        npix = 1
        for i in range(1, w):
            if line[i] == last and npix < 255:
                npix += 1
            else:
                cline.append(last)
                cline.append((0,0,npix))
                numpairs += 1
                last = line[i]
                npix = 1
        cline.append(last)
        cline.append((0,0,npix))
        numpairs += 1
        compressed.append(cline)
        maxlen = max(maxlen, len(cline))
    for line in compressed:
        if len(line) < maxlen:
            line += [(255,255,255)]*(maxlen-len(line))
    immagini.save(compressed, fileCompressed)
    return numpairs

########################################################################################

def decompress_image(fileCompressed, fileImageOut):
    '''Mettete il vostro codice qui'''
    img = immagini.load(fileCompressed)
    decompressed = []
    for cline in img:
        line = []
        while len(cline) > 1 and cline[0:2] != [(255,255,255),(255,255,255)]:
            color, (_,_,npix), *cline = cline
            line += [color]*npix
        decompressed.append(line)
    immagini.save(decompressed, fileImageOut)
    return len(decompressed[0])

########################################################################################

def color_histogram(fileImageIn, fileJsonOut):
    '''Mettete il vostro codice qui'''
    img = immagini.load(fileImageIn)
    colori = {}
    for line in img:
        for color in line:
            key = ";".join(map(str,color))
            if key in colori:
                colori[key] += 1
            else:
                colori[key] = 1
    with open(fileJsonOut, mode='w', encoding='utf8') as j:
        json.dump(colori, j, indent=4, sort_keys=True)
    return len(colori.keys())

########################################################################################

def paint_histogram(fileJsonIn, fileImageOut):
    '''Mettete il vostro codice qui'''
    with open(fileJsonIn, encoding='utf8') as f:
        isto = json.load(f)
        isto = { tuple(map(int,c.split(";"))) : v for c,v in isto.items()}
        istogramma = []
        maxw = 0
        for color in sorted(isto.keys()):
            numero = isto[color]
            istogramma.append([color] * numero)
            maxw = max(maxw, numero)
        for line in istogramma:
            if len(line) < maxw:
                line += [(0,0,0)] * (maxw - len(line))
        immagini.save(istogramma, fileImageOut)
        return maxw

########################################################################################
